/**
    Copyright (C) 2003  Michael Ahlberg, Måns Rullgård

    Permission is hereby granted, free of charge, to any person
    obtaining a copy of this software and associated documentation
    files (the "Software"), to deal in the Software without
    restriction, including without limitation the rights to use, copy,
    modify, merge, publish, distribute, sublicense, and/or sell copies
    of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be
    included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
**/

#define _LARGEFILE64_SOURCE

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <alloca.h>
#include <tctypes.h>
#include <tcstring.h>
#include <tcalloc.h>
#include <tcdirent.h>
#include <file_tc2.h>

typedef struct dir {
    char *files;
    int pos;
} dir_t;

#define min(a,b) ((a)<(b)?(a):(b))

extern int
dir_read(void *buf, size_t size, size_t count, url_t *u)
{
    dir_t *d = u->private;
    int bytes = min(size * count, u->size - d->pos);
    int n = bytes / size;
    bytes = n * size;
    memcpy(buf, d->files + d->pos, bytes);
    d->pos += bytes;
    return n;
}

extern int
dir_seek(url_t *u, int64_t offset, int how)
{
    dir_t *d = u->private;
    int64_t np;

    switch(how){
    case SEEK_SET:
	np = offset;
	break;
    case SEEK_CUR:
	np = d->pos + offset;
	break;
    case SEEK_END:
	np = u->size - offset;
	break;
    default:
	return -1;
    }

    if(np > u->size || np < 0)
	return -1;
    d->pos = np;

    return 0;
}

extern uint64_t
dir_tell(url_t *u)
{
    dir_t *d = u->private;
    return d->pos;
}

extern int
dir_close(url_t *u)
{
    dir_t *d = u->private;
    u->private = NULL;
    free(d->files);
    tcfree(u);
    return 0;
}

static void
dir_free(void *p)
{
    url_t *u = p;
    if(u->private){
	dir_t *d = u->private;
	u->private = NULL;
	free(d->files);
    }
}

extern url_t *
dir_open(char *url, char *mode)
{
    struct dirent **files;
    int i, n, size = 0, dnl;
    url_t *u;
    dir_t *d;
    char *p;

    int sel(const struct dirent *d){
	size += dnl + strlen(d->d_name) + 2;
	return strcmp(d->d_name, ".") && strcmp(d->d_name, "..");
    }

    if(strcmp(mode, "r"))
	return NULL;

    if(!strncmp(url, "file:", 5))
	url += 5;
    else if(!strncmp(url, "dir:", 4))
	url += 4;

    url = strdup(url);
    dnl = strlen(url);
    if(url[dnl - 1] == '/')
	url[--dnl] = 0;

    if((n = scandir(url, &files, sel, alphasort)) < 0)
	return NULL;

    d = calloc(1, sizeof(*d));
    d->files = malloc(size + 1);
    p = d->files;

    for(i = 0; i < n; i++){
	p += sprintf(p, "%s/%s\n", url, files[i]->d_name);
	free(files[i]);
    }
    size = p - d->files;
    free(files);
    free(url);

    u = tcallocdz(sizeof(*u), NULL, dir_free);
    u->read = dir_read;
    u->seek = dir_seek;
    u->tell = dir_tell;
    u->close = dir_close;
    u->size = size;
    u->private = d;

    return u;
}
