/**
    Copyright (C) 2004  Michael Ahlberg, Måns Rullgård

    Permission is hereby granted, free of charge, to any person
    obtaining a copy of this software and associated documentation
    files (the "Software"), to deal in the Software without
    restriction, including without limitation the rights to use, copy,
    modify, merge, publish, distribute, sublicense, and/or sell copies
    of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be
    included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
**/

#include <stdio.h>
#include <string.h>
#include <image.h>
#include <image_tc2.h>

#ifdef HAVE_LIBMAGIC
#include <magic.h>
static magic_t file_magic;
#define MAGIC_SIZE 32
#endif

extern image_t *
read_img(url_t *u, image_params_t *params)
{
    char *m = NULL;
    image_read_t iread;

#ifdef HAVE_LIBMAGIC
    char buf[MAGIC_SIZE];
    const char *mg;
    int mgs;

    mgs = u->read(buf, 1, MAGIC_SIZE, u);
    u->seek(u, 0, SEEK_SET);
    if(mgs < MAGIC_SIZE)
	return NULL;
    mg = magic_buffer(file_magic, buf, mgs);

    tc2_print("IMAGE", TC2_PRINT_DEBUG, "detected %s\n", mg);

    if(mg){
	int e;
	m = strdup(mg);
	e = strcspn(m, " \t;");
	m[e] = 0;
	if(strncmp(m, "image/", 6)){
	    free(m);
	    m = NULL;
	}
    }
#else
#warning "libmagic is required"
#endif

    if(!m)
	return NULL;

    if(!(iread = tc2_get_symbol(m, "read")))
	return NULL;

    return iread(u, params);
}

extern int
img_init(char *p)
{
#ifdef HAVE_LIBMAGIC
    file_magic = magic_open(MAGIC_MIME);
    magic_load(file_magic, NULL);
#endif

    return 0;
}

extern int
img_shdn(void)
{
#ifdef HAVE_LIBMAGIC
    magic_close(file_magic);
#endif
    return 0;
}
