/**
    Copyright (C) 2001-2004  Michael Ahlberg, Måns Rullgård

    Permission is hereby granted, free of charge, to any person
    obtaining a copy of this software and associated documentation
    files (the "Software"), to deal in the Software without
    restriction, including without limitation the rights to use, copy,
    modify, merge, publish, distribute, sublicense, and/or sell copies
    of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be
    included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
**/

#include <image.h>
#include <stdio.h>
#include <jpeglib.h>
#include <setjmp.h>
#include <string.h>
#include <tcalloc.h>
#include <jpeg_tc2.h>
#include <jpegmod.h>

/* Error handling */
struct my_error_mgr {
    struct jpeg_error_mgr pub;	/* "public" fields */
    jmp_buf setjmp_buffer;	/* for return to caller */
};

typedef struct my_error_mgr * my_error_ptr;

/*
 * Here's the routine that will replace the standard error_exit method:
 */

static void
my_error_exit (j_common_ptr cinfo)
{
    /* cinfo->err really points to a my_error_mgr struct, so coerce pointer */
    my_error_ptr myerr = (my_error_ptr) cinfo->err;
    
    /* Always display the message. */
    /* We could postpone this until after returning, if we chose. */
    (*cinfo->err->output_message) (cinfo);

    /* Return control to the setjmp point */
    longjmp(myerr->setjmp_buffer, 1);
}

static void
free_jpeg(void *p)
{
    image_t *img = p;
    free(img->data[0]);
}

extern image_t *
read_jpg(url_t *u, image_params_t *ip)
{
    struct jpeg_decompress_struct cinfo;
    struct my_error_mgr jerr;
    JSAMPARRAY buffer;            /* Output row buffer */
    int row_stride;               /* physical row width in output buffer */
    unsigned char *data;
    image_t *im;

    /* We set up the normal JPEG error routines, then override error_exit. */
    cinfo.err = jpeg_std_error(&jerr.pub);
    jerr.pub.error_exit = my_error_exit;

    /* Establish the setjmp return context for my_error_exit to use. */
    if (setjmp(jerr.setjmp_buffer)) {
	/* If we get here, the JPEG code has signaled an error. */
	jpeg_destroy_decompress(&cinfo);
	return NULL;
    }

    jpeg_create_decompress(&cinfo);

    io_src(&cinfo, u);

    jpeg_read_header(&cinfo, FALSE);
    jpeg_start_decompress(&cinfo);
    
    /* JSAMPLEs per row in output buffer */
    row_stride = cinfo.output_width * cinfo.output_components;

    im = tcallocdz(sizeof(*im), NULL, free_jpeg);
    im->params.width[0] = cinfo.output_width;
    im->params.height[0] = cinfo.output_height;
    im->params.pixel_type = IMAGE_COLOR_RGB_ALPHA;
    im->params.pixelsize = 4;
    im->params.mask[0] = 0xff0000;
    im->params.mask[1] = 0xff00;
    im->params.mask[2] = 0xff;
    im->params.mask[3] = 0xff000000;

    data = malloc(cinfo.output_height * cinfo.output_width * 4);
    im->data[0] = data;
    im->linesize[0] = cinfo.output_width * 4;

    buffer = cinfo.mem->alloc_sarray((j_common_ptr) &cinfo, JPOOL_IMAGE,
				     row_stride, 1);

    while (cinfo.output_scanline < cinfo.output_height) {
	int i;
	jpeg_read_scanlines(&cinfo, buffer, 1);
	for(i = 0; i < cinfo.output_width; i++){
	    *data++ = buffer[0][i*3+2];
	    *data++ = buffer[0][i*3+1];
	    *data++ = buffer[0][i*3];
	    *data++ = 255;
	}
    }

    jpeg_finish_decompress(&cinfo);
    jpeg_destroy_decompress(&cinfo);

    return im;
}
